import checkov.terraform.tag_providers.aws
import checkov.terraform.tag_providers.azure
import checkov.terraform.tag_providers.gcp

provider_tag_mapping = {
    'aws': aws.get_resource_tags,
    'azure': azure.get_resource_tags,
    'gcp': gcp.get_resource_tags
}


def get_resource_tags(resource_type, entity_config):
    if '_' not in resource_type:
        return None  # probably not a resource block
    provider = resource_type[:resource_type.index('_')]
    provider_tag_function = provider_tag_mapping.get(provider)
    if provider_tag_function:
        return provider_tag_function(entity_config)
    else:
        return None



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SecurityGroupRuleDescription(BaseResourceCheck):
    def __init__(self):
        name = "Ensure every security groups rule has a description"
        id = "CKV_AWS_23"
        supported_resource = [
            'aws_security_group',
            'aws_security_group_rule',
            'aws_db_security_group',
            'aws_elasticache_security_group',
            'aws_redshift_security_group',
        ]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def scan_resource_conf(self, conf):
        """
            Looks for description at security group  rules :
            https://www.terraform.io/docs/providers/aws/r/security_group.html
        :param conf: aws_security_group configuration
        :return: <CheckResult>
        """
        if 'description' in conf.keys():
            if conf['description']:
                self.evaluated_keys = 'description'
                return CheckResult.PASSED
        egress_result = self.check_rule(rule_type='egress', conf=conf)
        ingress_result = self.check_rule(rule_type='ingress', conf=conf)
        if egress_result == CheckResult.PASSED and ingress_result == CheckResult.PASSED:
            return CheckResult.PASSED
        return CheckResult.FAILED

    def check_rule(self, rule_type, conf):
        if rule_type in conf.keys():
            for rule in conf[rule_type]:
                if isinstance(rule, dict):
                    if 'description' not in rule.keys() or not rule['description']:
                        self.evaluated_keys.append(f'{rule_type}/{conf[rule_type].index(rule)}')
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = SecurityGroupRuleDescription()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.secrets import string_has_secrets, AWS
from checkov.common.util.type_forcers import force_list


class LambdaEnvironmentCredentials(BaseResourceCheck):

    def __init__(self):
        name = "Ensure no hard-coded secrets exists in lambda environment"
        id = "CKV_AWS_45"
        supported_resources = ['aws_lambda_function']
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        self.evaluated_keys = 'environment/[0]/variables'
        if 'environment' in conf.keys():
            if isinstance(conf['environment'][0], dict):
                if 'variables' in conf['environment'][0]:
                    if isinstance(force_list(conf['environment'][0]['variables'])[0], dict):
                        # variables can be a string, which in this case it points to a variable
                        for values in list(force_list(conf['environment'][0]['variables'])[0].values()):
                            for value in list(filter(lambda value: isinstance(value, str), force_list(values))):
                                if string_has_secrets(value, AWS):
                                    return CheckResult.FAILED
        return CheckResult.PASSED


check = LambdaEnvironmentCredentials()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ELBv2AccessLogs(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure the ELBv2 (Application/Network) has access logging enabled"
        id = "CKV_AWS_91"
        supported_resources = ['aws_lb', 'aws_alb']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'access_logs/0/enabled/0'


check = ELBv2AccessLogs()
